@@ -1,6 +1,7 @@
 /*
      This file is part of libmicrohttpd
      Copyright (C) 2007, 2008 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2014-2022 Evgeny Grin (Karlson2k)
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -20,12 +21,14 @@
  * @file fileserver_example_external_select.c
  * @brief minimal example for how to use libmicrohttpd to server files
  * @author Christian Grothoff
+ * @author Karlson2k (Evgeny Grin)
  */
 
 #include "platform.h"
 #include <microhttpd.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <errno.h>
 
 #define PAGE \
   "<html><head><title>File not found</title></head><body>File not found</body></html>"
@@ -33,10 +36,19 @@
 static ssize_t
 file_reader (void *cls, uint64_t pos, char *buf, size_t max)
 {
-  FILE *file = cls;
+  FILE *file = (FILE *) cls;
+  size_t bytes_read;
 
-  (void) fseek (file, pos, SEEK_SET);
-  return fread (buf, 1, max, file);
+  /* 'fseek' may not support files larger 2GiB, depending on platform.
+   * For production code, make sure that 'pos' has valid values, supported by
+   * 'fseek', or use 'fseeko' or similar function. */
+  if (0 != fseek (file, (long) pos, SEEK_SET))
+    return MHD_CONTENT_READER_END_WITH_ERROR;
+  bytes_read = fread (buf, 1, max, file);
+  if (0 == bytes_read)
+    return (0 != ferror (file)) ? MHD_CONTENT_READER_END_WITH_ERROR :
+           MHD_CONTENT_READER_END_OF_STREAM;
+  return (ssize_t) bytes_read;
 }
 
 
@@ -55,7 +67,7 @@ ahc_echo (void *cls,
           const char *method,
           const char *version,
           const char *upload_data,
-          size_t *upload_data_size, void **ptr)
+          size_t *upload_data_size, void **req_cls)
 {
   static int aptr;
   struct MHD_Response *response;
@@ -70,13 +82,13 @@ ahc_echo (void *cls,
 
   if (0 != strcmp (method, MHD_HTTP_METHOD_GET))
     return MHD_NO;              /* unexpected method */
-  if (&aptr != *ptr)
+  if (&aptr != *req_cls)
   {
     /* do never respond on first call */
-    *ptr = &aptr;
+    *req_cls = &aptr;
     return MHD_YES;
   }
-  *ptr = NULL;                  /* reset when done */
+  *req_cls = NULL;                  /* reset when done */
 
   file = fopen (&url[1], "rb");
   if (NULL != file)
@@ -98,15 +110,15 @@ ahc_echo (void *cls,
 
   if (NULL == file)
   {
-    response = MHD_create_response_from_buffer (strlen (PAGE),
-                                                (void *) PAGE,
-                                                MHD_RESPMEM_PERSISTENT);
+    response = MHD_create_response_from_buffer_static (strlen (PAGE),
+                                                       PAGE);
     ret = MHD_queue_response (connection, MHD_HTTP_NOT_FOUND, response);
     MHD_destroy_response (response);
   }
   else
   {
-    response = MHD_create_response_from_callback (buf.st_size, 32 * 1024,       /* 32k page size */
+    response = MHD_create_response_from_callback ((size_t) buf.st_size,
+                                                  32 * 1024, /* 32k page size */
                                                   &file_reader,
                                                   file,
                                                   &free_callback);
@@ -133,22 +145,37 @@ main (int argc, char *const *argv)
   fd_set ws;
   fd_set es;
   MHD_socket max;
-  MHD_UNSIGNED_LONG_LONG mhd_timeout;
+  uint64_t mhd_timeout;
+  int port;
 
   if (argc != 3)
   {
     printf ("%s PORT SECONDS-TO-RUN\n", argv[0]);
     return 1;
   }
+  port = atoi (argv[1]);
+  if ( (1 > port) || (port > 65535) )
+  {
+    fprintf (stderr,
+             "Port must be a number between 1 and 65535.\n");
+    return 1;
+  }
+
   d = MHD_start_daemon (MHD_USE_ERROR_LOG,
-                        atoi (argv[1]),
-                        NULL, NULL, &ahc_echo, PAGE, MHD_OPTION_END);
+                        (uint16_t) port,
+                        NULL, NULL, &ahc_echo, NULL,
+                        MHD_OPTION_APP_FD_SETSIZE, (int) FD_SETSIZE,
+                        MHD_OPTION_END);
   if (d == NULL)
     return 1;
   end = time (NULL) + atoi (argv[2]);
   while ((t = time (NULL)) < end)
   {
+#if ! defined(_WIN32) || defined(__CYGWIN__)
     tv.tv_sec = end - t;
+#else  /* Native W32 */
+    tv.tv_sec = (long) (end - t);
+#endif /* Native W32 */
     tv.tv_usec = 0;
     max = 0;
     FD_ZERO (&rs);
@@ -156,15 +183,19 @@ main (int argc, char *const *argv)
     FD_ZERO (&es);
     if (MHD_YES != MHD_get_fdset (d, &rs, &ws, &es, &max))
       break; /* fatal internal error */
-    if (MHD_get_timeout (d, &mhd_timeout) == MHD_YES)
+    if (MHD_get_timeout64 (d, &mhd_timeout) == MHD_YES)
     {
-      if (((MHD_UNSIGNED_LONG_LONG) tv.tv_sec) < mhd_timeout / 1000LL)
+      if (((uint64_t) tv.tv_sec) < mhd_timeout / 1000LL)
       {
-        tv.tv_sec = mhd_timeout / 1000LL;
-        tv.tv_usec = (mhd_timeout - (tv.tv_sec * 1000LL)) * 1000LL;
+#if ! defined(_WIN32) || defined(__CYGWIN__)
+        tv.tv_sec = (time_t) (mhd_timeout / 1000LL);
+#else  /* Native W32 */
+        tv.tv_sec = (long) (mhd_timeout / 1000LL);
+#endif /* Native W32 */
+        tv.tv_usec = ((long) (mhd_timeout % 1000)) * 1000;
       }
     }
-    if (-1 == select (max + 1, &rs, &ws, &es, &tv))
+    if (-1 == select ((int) max + 1, &rs, &ws, &es, &tv))
     {
       if (EINTR != errno)
         abort ();
