@@ -2,6 +2,7 @@
      This file is part of libmicrohttpd
      Copyright (C) 2020 Christian Grothoff, Silvio Clecio (and other
      contributing authors)
+     Copyright (C) 2020-2022 Evgeny Grin (Karlson2k)
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -22,11 +23,13 @@
  * @file websocket_threaded_example.c
  * @brief example for how to provide a tiny threaded websocket server
  * @author Silvio Clecio (silvioprog)
+ * @author Karlson2k (Evgeny Grin)
  */
 
 /* TODO: allow to send large messages. */
 
 #include "platform.h"
+#include <errno.h>
 #include <pthread.h>
 #include <microhttpd.h>
 
@@ -36,7 +39,7 @@
   "<title>WebSocket chat</title>\n"                                           \
   "<script>\n"                                                                \
   "document.addEventListener('DOMContentLoaded', function() {\n"              \
-  "  const ws = new WebSocket('ws:/" "/ ' + window.location.host);\n"         \
+  "  const ws = new WebSocket('ws:/" "/' + window.location.host);\n"          \
   "  const btn = document.getElementById('send');\n"                          \
   "  const msg = document.getElementById('msg');\n"                           \
   "  const log = document.getElementById('log');\n"                           \
@@ -149,9 +152,9 @@ SHA1ProcessMessageBlock (struct SHA1Context *context)
 
   for (i = 0; i < 16; i++)
   {
-    W[i] = context->message_block[i * 4] << 24;
-    W[i] |= context->message_block[i * 4 + 1] << 16;
-    W[i] |= context->message_block[i * 4 + 2] << 8;
+    W[i] = ((uint32_t) context->message_block[i * 4]) << 24;
+    W[i] |= ((uint32_t) context->message_block[i * 4 + 1]) << 16;
+    W[i] |= ((uint32_t) context->message_block[i * 4 + 2]) << 8;
     W[i] |= context->message_block[i * 4 + 3];
   }
   for (i = 16; i < 80; i++)
@@ -235,14 +238,14 @@ SHA1PadMessage (struct SHA1Context *context)
       context->message_block[context->message_block_index++] = 0;
     }
   }
-  context->message_block[56] = context->length_high >> 24;
-  context->message_block[57] = context->length_high >> 16;
-  context->message_block[58] = context->length_high >> 8;
-  context->message_block[59] = context->length_high;
-  context->message_block[60] = context->length_low >> 24;
-  context->message_block[61] = context->length_low >> 16;
-  context->message_block[62] = context->length_low >> 8;
-  context->message_block[63] = context->length_low;
+  context->message_block[56] = (unsigned char) (context->length_high >> 24);
+  context->message_block[57] = (unsigned char) (context->length_high >> 16);
+  context->message_block[58] = (unsigned char) (context->length_high >> 8);
+  context->message_block[59] = (unsigned char) (context->length_high);
+  context->message_block[60] = (unsigned char) (context->length_low >> 24);
+  context->message_block[61] = (unsigned char) (context->length_low >> 16);
+  context->message_block[62] = (unsigned char) (context->length_low >> 8);
+  context->message_block[63] = (unsigned char) (context->length_low);
   SHA1ProcessMessageBlock (context);
 }
 
@@ -296,7 +299,8 @@ SHA1Result (struct SHA1Context *context, unsigned char
   for (i = 0; i < SHA1HashSize; ++i)
   {
     Message_Digest[i]
-      = context->intermediate_hash[i >> 2] >> 8 * (3 - (i & 0x03));
+      = (unsigned char) (context->intermediate_hash[i >> 2]
+                         >> 8 * (3 - (i & 0x03)));
   }
   return SHA1_RESULT_SUCCESS;
 }
@@ -352,7 +356,7 @@ SHA1Input (struct SHA1Context *context, const unsigned char *message_array,
 /********** begin Base64 **********/
 
 
-ssize_t
+static ssize_t
 BASE64Encode (const void *in, size_t len, char **output)
 {
 #define FILLCHAR '='
@@ -378,15 +382,15 @@ BASE64Encode (const void *in, size_t len, char **output)
     c = (data[i] << 4) & 0x3F;
     if (++i < len)
     {
-      c |= (data[i] >> 4) & 0x0F;
+      c = (char) (c | ((data[i] >> 4) & 0x0F));
     }
     opt[ret++] = cvt[(int) c];
     if (i < len)
     {
-      c = (data[i] << 2) & 0x3F;
+      c = (char) (c | ((data[i] << 2) & 0x3F));
       if (++i < len)
       {
-        c |= (data[i] >> 6) & 0x03;
+        c = (char) (c | ((data[i] >> 6) & 0x03));
       }
       opt[ret++] = cvt[(int) c];
     }
@@ -434,8 +438,8 @@ send_chat_page (struct MHD_Connection *con)
   struct MHD_Response *res;
   enum MHD_Result ret;
 
-  res = MHD_create_response_from_buffer (strlen (CHAT_PAGE), (void *) CHAT_PAGE,
-                                         MHD_RESPMEM_PERSISTENT);
+  res = MHD_create_response_from_buffer_static (strlen (CHAT_PAGE),
+                                                (const void *) CHAT_PAGE);
   ret = MHD_queue_response (con, MHD_HTTP_OK, res);
   MHD_destroy_response (res);
   return ret;
@@ -448,9 +452,9 @@ send_bad_request (struct MHD_Connection *con)
   struct MHD_Response *res;
   enum MHD_Result ret;
 
-  res = MHD_create_response_from_buffer (strlen (BAD_REQUEST_PAGE),
-                                         (void *) BAD_REQUEST_PAGE,
-                                         MHD_RESPMEM_PERSISTENT);
+  res =
+    MHD_create_response_from_buffer_static (strlen (BAD_REQUEST_PAGE),
+                                            (const void *) BAD_REQUEST_PAGE);
   ret = MHD_queue_response (con, MHD_HTTP_BAD_REQUEST, res);
   MHD_destroy_response (res);
   return ret;
@@ -463,11 +467,17 @@ send_upgrade_required (struct MHD_Connection *con)
   struct MHD_Response *res;
   enum MHD_Result ret;
 
-  res = MHD_create_response_from_buffer (strlen (UPGRADE_REQUIRED_PAGE),
-                                         (void *) UPGRADE_REQUIRED_PAGE,
-                                         MHD_RESPMEM_PERSISTENT);
-  MHD_add_response_header (res, MHD_HTTP_HEADER_SEC_WEBSOCKET_VERSION,
-                           WS_SEC_WEBSOCKET_VERSION);
+  res =
+    MHD_create_response_from_buffer_static (strlen (UPGRADE_REQUIRED_PAGE),
+                                            (const void *)
+                                            UPGRADE_REQUIRED_PAGE);
+  if (MHD_YES !=
+      MHD_add_response_header (res, MHD_HTTP_HEADER_SEC_WEBSOCKET_VERSION,
+                               WS_SEC_WEBSOCKET_VERSION))
+  {
+    MHD_destroy_response (res);
+    return MHD_NO;
+  }
   ret = MHD_queue_response (con, MHD_HTTP_UPGRADE_REQUIRED, res);
   MHD_destroy_response (res);
   return ret;
@@ -538,9 +548,13 @@ send_all (MHD_socket sock, const unsigned char *buf, size_t len)
   ssize_t ret;
   size_t off;
 
-  for (off = 0; off < len; off += ret)
+  for (off = 0; off < len; off += (size_t) ret)
   {
+#if ! defined(_WIN32) || defined(__CYGWIN__)
     ret = send (sock, (const void *) &buf[off], len - off, 0);
+#else  /* Native W32 */
+    ret = send (sock, (const void *) &buf[off], (int) (len - off), 0);
+#endif /* Native W32 */
     if (0 > ret)
     {
       if (EAGAIN == errno)
@@ -559,14 +573,14 @@ send_all (MHD_socket sock, const unsigned char *buf, size_t len)
 }
 
 
-static int
+static ssize_t
 ws_send_frame (MHD_socket sock, const char *msg, size_t length)
 {
   unsigned char *response;
   unsigned char frame[10];
   unsigned char idx_first_rdata;
-  int idx_response;
-  int output;
+  size_t idx_response;
+  size_t output;
   MHD_socket isock;
   size_t i;
 
@@ -611,12 +625,13 @@ ws_send_frame (MHD_socket sock, const char *msg, size_t length)
   }
   for (i = 0; i < length; i++)
   {
-    response[idx_response] = msg[i];
+    response[idx_response] = (unsigned char) msg[i];
     idx_response++;
   }
   response[idx_response] = '\0';
   output = 0;
-  pthread_mutex_lock (&MUTEX);
+  if (0 != pthread_mutex_lock (&MUTEX))
+    abort ();
   for (i = 0; i < MAX_CLIENTS; i++)
   {
     isock = CLIENT_SOCKS[i];
@@ -625,9 +640,10 @@ ws_send_frame (MHD_socket sock, const char *msg, size_t length)
       output += send_all (isock, response, idx_response);
     }
   }
-  pthread_mutex_unlock (&MUTEX);
+  if (0 != pthread_mutex_unlock (&MUTEX))
+    abort ();
   free (response);
-  return output;
+  return (ssize_t) output;
 }
 
 
@@ -640,7 +656,7 @@ ws_receive_frame (unsigned char *frame, ssize_t *length, int *type)
   unsigned char flength;
   unsigned char idx_first_mask;
   unsigned char idx_first_data;
-  ssize_t data_length;
+  size_t data_length;
   int i;
   int j;
 
@@ -659,8 +675,8 @@ ws_receive_frame (unsigned char *frame, ssize_t *length, int *type)
     {
       idx_first_mask = 10;
     }
-    idx_first_data = idx_first_mask + 4;
-    data_length = *length - idx_first_data;
+    idx_first_data = (unsigned char) (idx_first_mask + 4);
+    data_length = (size_t) *length - idx_first_data;
     masks[0] = frame[idx_first_mask + 0];
     masks[1] = frame[idx_first_mask + 1];
     masks[2] = frame[idx_first_mask + 2];
@@ -672,7 +688,7 @@ ws_receive_frame (unsigned char *frame, ssize_t *length, int *type)
       {
         msg[j] = frame[i] ^ masks[j % 4];
       }
-      *length = data_length;
+      *length = (ssize_t) data_length;
       msg[j] = '\0';
     }
   }
@@ -695,12 +711,9 @@ run_usock (void *cls)
   struct MHD_UpgradeResponseHandle *urh = ws->urh;
   unsigned char buf[2048];
   unsigned char *msg;
-  char client[20];
   char *text;
   ssize_t got;
-  size_t size;
   int type;
-  int sent;
   int i;
 
   make_blocking (ws->sock);
@@ -718,19 +731,26 @@ run_usock (void *cls)
     }
     if (type == WS_OPCODE_TEXT_FRAME)
     {
-      size = sprintf (client, "User#%d: ", (int) ws->sock);
-      size += got;
-      text = malloc (size);
-      if (NULL != text)
+      ssize_t sent;
+      int buf_size;
+      buf_size = snprintf (NULL, 0, "User#%d: %s", (int) ws->sock, msg);
+      if (0 < buf_size)
       {
-        sprintf (text, "%s%s", client, msg);
-        sent = ws_send_frame (ws->sock, text, size);
-        free (text);
+        text = malloc ((size_t) buf_size + 1);
+        if (NULL != text)
+        {
+          if (snprintf (text, (size_t) buf_size + 1,
+                        "User#%d: %s", (int) ws->sock, msg) == buf_size)
+            sent = ws_send_frame (ws->sock, text, (size_t) buf_size);
+          else
+            sent = -1;
+          free (text);
+        }
+        else
+          sent = -1;
       }
       else
-      {
         sent = -1;
-      }
       free (msg);
       if (-1 == sent)
       {
@@ -746,7 +766,8 @@ run_usock (void *cls)
       }
     }
   }
-  pthread_mutex_lock (&MUTEX);
+  if (0 != pthread_mutex_lock (&MUTEX))
+    abort ();
   for (i = 0; i < MAX_CLIENTS; i++)
   {
     if (CLIENT_SOCKS[i] == ws->sock)
@@ -755,7 +776,8 @@ run_usock (void *cls)
       break;
     }
   }
-  pthread_mutex_unlock (&MUTEX);
+  if (0 != pthread_mutex_unlock (&MUTEX))
+    abort ();
   free (ws);
   MHD_upgrade_action (urh, MHD_UPGRADE_ACTION_CLOSE);
   return NULL;
@@ -763,7 +785,7 @@ run_usock (void *cls)
 
 
 static void
-uh_cb (void *cls, struct MHD_Connection *con, void *con_cls,
+uh_cb (void *cls, struct MHD_Connection *con, void *req_cls,
        const char *extra_in, size_t extra_in_size, MHD_socket sock,
        struct MHD_UpgradeResponseHandle *urh)
 {
@@ -774,7 +796,7 @@ uh_cb (void *cls, struct MHD_Connection *con, void *con_cls,
 
   (void) cls;            /* Unused. Silent compiler warning. */
   (void) con;            /* Unused. Silent compiler warning. */
-  (void) con_cls;        /* Unused. Silent compiler warning. */
+  (void) req_cls;        /* Unused. Silent compiler warning. */
   (void) extra_in;       /* Unused. Silent compiler warning. */
   (void) extra_in_size;  /* Unused. Silent compiler warning. */
 
@@ -785,7 +807,8 @@ uh_cb (void *cls, struct MHD_Connection *con, void *con_cls,
   ws->sock = sock;
   ws->urh = urh;
   sock_overflow = MHD_YES;
-  pthread_mutex_lock (&MUTEX);
+  if (0 != pthread_mutex_lock (&MUTEX))
+    abort ();
   for (i = 0; i < MAX_CLIENTS; i++)
   {
     if (MHD_INVALID_SOCKET == CLIENT_SOCKS[i])
@@ -795,7 +818,8 @@ uh_cb (void *cls, struct MHD_Connection *con, void *con_cls,
       break;
     }
   }
-  pthread_mutex_unlock (&MUTEX);
+  if (0 != pthread_mutex_unlock (&MUTEX))
+    abort ();
   if (sock_overflow)
   {
     free (ws);
@@ -814,7 +838,7 @@ uh_cb (void *cls, struct MHD_Connection *con, void *con_cls,
 static enum MHD_Result
 ahc_cb (void *cls, struct MHD_Connection *con, const char *url,
         const char *method, const char *version, const char *upload_data,
-        size_t *upload_data_size, void **ptr)
+        size_t *upload_data_size, void **req_cls)
 {
   struct MHD_Response *res;
   const char *upg_header;
@@ -830,12 +854,12 @@ ahc_cb (void *cls, struct MHD_Connection *con, const char *url,
   (void) upload_data;       /* Unused. Silent compiler warning. */
   (void) upload_data_size;  /* Unused. Silent compiler warning. */
 
-  if (NULL == *ptr)
+  if (NULL == *req_cls)
   {
-    *ptr = (void *) 1;
+    *req_cls = (void *) 1;
     return MHD_YES;
   }
-  *ptr = NULL;
+  *req_cls = NULL;
   upg_header = MHD_lookup_connection_value (con, MHD_HEADER_KIND,
                                             MHD_HTTP_HEADER_UPGRADE);
   con_header = MHD_lookup_connection_value (con, MHD_HEADER_KIND,
@@ -849,18 +873,19 @@ ahc_cb (void *cls, struct MHD_Connection *con, const char *url,
   {
     return send_bad_request (con);
   }
-  ws_version_header = MHD_lookup_connection_value (
-    con, MHD_HEADER_KIND, MHD_HTTP_HEADER_SEC_WEBSOCKET_VERSION);
+  ws_version_header =
+    MHD_lookup_connection_value (con, MHD_HEADER_KIND,
+                                 MHD_HTTP_HEADER_SEC_WEBSOCKET_VERSION);
   if ((NULL == ws_version_header)
       || (0 != strcmp (ws_version_header, WS_SEC_WEBSOCKET_VERSION)))
   {
     return send_upgrade_required (con);
   }
-  ret = MHD_lookup_connection_value_n (
-    con, MHD_HEADER_KIND,
-    MHD_HTTP_HEADER_SEC_WEBSOCKET_KEY,
-    strlen (MHD_HTTP_HEADER_SEC_WEBSOCKET_KEY),
-    &ws_key_header, &key_size);
+  ret = MHD_lookup_connection_value_n (con, MHD_HEADER_KIND,
+                                       MHD_HTTP_HEADER_SEC_WEBSOCKET_KEY,
+                                       strlen (
+                                         MHD_HTTP_HEADER_SEC_WEBSOCKET_KEY),
+                                       &ws_key_header, &key_size);
   if ((MHD_NO == ret) || (key_size != WS_KEY_LEN))
   {
     return send_bad_request (con);
@@ -871,10 +896,21 @@ ahc_cb (void *cls, struct MHD_Connection *con, const char *url,
     return ret;
   }
   res = MHD_create_response_for_upgrade (&uh_cb, NULL);
-  MHD_add_response_header (res, MHD_HTTP_HEADER_UPGRADE, WS_UPGRADE_VALUE);
-  MHD_add_response_header (res, MHD_HTTP_HEADER_SEC_WEBSOCKET_ACCEPT,
-                           ws_ac_value);
+  if (MHD_YES !=
+      MHD_add_response_header (res, MHD_HTTP_HEADER_SEC_WEBSOCKET_ACCEPT,
+                               ws_ac_value))
+  {
+    free (ws_ac_value);
+    MHD_destroy_response (res);
+    return MHD_NO;
+  }
   free (ws_ac_value);
+  if (MHD_YES !=
+      MHD_add_response_header (res, MHD_HTTP_HEADER_UPGRADE, WS_UPGRADE_VALUE))
+  {
+    MHD_destroy_response (res);
+    return MHD_NO;
+  }
   ret = MHD_queue_response (con, MHD_HTTP_SWITCHING_PROTOCOLS, res);
   MHD_destroy_response (res);
   return ret;
@@ -885,18 +921,19 @@ int
 main (int argc, char *const *argv)
 {
   struct MHD_Daemon *d;
-  uint16_t port;
+  unsigned int port;
   size_t i;
-
-  if (argc != 2)
+  if ( (argc != 2) ||
+       (1 != sscanf (argv[1], "%u", &port)) ||
+       (65535 < port) )
   {
     printf ("%s PORT\n", argv[0]);
     return 1;
   }
-  port = atoi (argv[1]);
   d = MHD_start_daemon (MHD_ALLOW_UPGRADE | MHD_USE_AUTO_INTERNAL_THREAD
                         | MHD_USE_ERROR_LOG,
-                        port, NULL, NULL, &ahc_cb, &port, MHD_OPTION_END);
+                        (uint16_t) port, NULL, NULL,
+                        &ahc_cb, NULL, MHD_OPTION_END);
   if (NULL == d)
     return 1;
   for (i = 0; i < sizeof(CLIENT_SOCKS) / sizeof(CLIENT_SOCKS[0]); ++i)
