@@ -1,6 +1,7 @@
 /*
      This file is part of libmicrohttpd
      Copyright (C) 2007 Daniel Pittman and Christian Grothoff
+     Copyright (C) 2015-2023 Evgeny Grin (Karlson2k)
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -22,6 +23,7 @@
  * @brief  internal shared structures
  * @author Daniel Pittman
  * @author Christian Grothoff
+ * @author Karlson2k (Evgeny Grin)
  */
 
 #include "internal.h"
@@ -41,22 +43,22 @@ MHD_state_to_string (enum MHD_CONNECTION_STATE state)
     return "connection init";
   case MHD_CONNECTION_REQ_LINE_RECEIVING:
     return "receiving request line";
-  case MHD_CONNECTION_URL_RECEIVED:
-    return "connection url received";
-  case MHD_CONNECTION_HEADER_PART_RECEIVED:
-    return "header partially received";
+  case MHD_CONNECTION_REQ_LINE_RECEIVED:
+    return "request line received";
+  case MHD_CONNECTION_REQ_HEADERS_RECEIVING:
+    return "headers receiving";
   case MHD_CONNECTION_HEADERS_RECEIVED:
     return "headers received";
   case MHD_CONNECTION_HEADERS_PROCESSED:
     return "headers processed";
   case MHD_CONNECTION_CONTINUE_SENDING:
     return "continue sending";
-  case MHD_CONNECTION_CONTINUE_SENT:
-    return "continue sent";
+  case MHD_CONNECTION_BODY_RECEIVING:
+    return "body receiving";
   case MHD_CONNECTION_BODY_RECEIVED:
     return "body received";
-  case MHD_CONNECTION_FOOTER_PART_RECEIVED:
-    return "footer partially received";
+  case MHD_CONNECTION_FOOTERS_RECEIVING:
+    return "footers receiving";
   case MHD_CONNECTION_FOOTERS_RECEIVED:
     return "footers received";
   case MHD_CONNECTION_FULL_REQ_RECEIVED:
@@ -75,12 +77,12 @@ MHD_state_to_string (enum MHD_CONNECTION_STATE state)
     return "chunked body unready";
   case MHD_CONNECTION_CHUNKED_BODY_READY:
     return "chunked body ready";
-  case MHD_CONNECTION_BODY_SENT:
-    return "body sent";
+  case MHD_CONNECTION_CHUNKED_BODY_SENT:
+    return "chunked body sent";
   case MHD_CONNECTION_FOOTERS_SENDING:
     return "footers sending";
-  case MHD_CONNECTION_FOOTERS_SENT:
-    return "footers sent";
+  case MHD_CONNECTION_FULL_REPLY_SENT:
+    return "reply sent completely";
   case MHD_CONNECTION_CLOSED:
     return "closed";
   default:
@@ -135,44 +137,17 @@ MHD_unescape_plus (char *arg)
 
 /**
  * Process escape sequences ('%HH') Updates val in place; the
- * result should be UTF-8 encoded and cannot be larger than the input.
- * The result must also still be 0-terminated.
+ * result cannot be larger than the input.
+ * The result is still be 0-terminated.
  *
  * @param val value to unescape (modified in the process)
- * @return length of the resulting val (strlen(val) maybe
+ * @return length of the resulting val (`strlen(val)` may be
  *  shorter afterwards due to elimination of escape sequences)
  */
-size_t
+_MHD_EXTERN size_t
 MHD_http_unescape (char *val)
 {
-  char *rpos = val;
-  char *wpos = val;
-
-  while ('\0' != *rpos)
-  {
-    uint32_t num;
-    switch (*rpos)
-    {
-    case '%':
-      if (2 == MHD_strx_to_uint32_n_ (rpos + 1,
-                                      2,
-                                      &num))
-      {
-        *wpos = (char) ((unsigned char) num);
-        wpos++;
-        rpos += 3;
-        break;
-      }
-    /* TODO: add bad sequence handling */
-    /* intentional fall through! */
-    default:
-      *wpos = *rpos;
-      wpos++;
-      rpos++;
-    }
-  }
-  *wpos = '\0'; /* add 0-terminator */
-  return wpos - val;
+  return MHD_str_pct_decode_in_place_lenient_ (val, NULL);
 }
 
 
@@ -185,7 +160,7 @@ MHD_http_unescape (char *val)
  * @param[in,out] args argument URI string (after "?" in URI),
  *        clobbered in the process!
  * @param cb function to call on each key-value pair found
- * @param[out] num_headers set to the number of headers found
+ * @param cls the iterator context
  * @return #MHD_NO on failure (@a cb returned #MHD_NO),
  *         #MHD_YES for success (parsing succeeded, @a cb always
  *                               returned #MHD_YES)
@@ -195,13 +170,12 @@ MHD_parse_arguments_ (struct MHD_Connection *connection,
                       enum MHD_ValueKind kind,
                       char *args,
                       MHD_ArgumentIterator_ cb,
-                      unsigned int *num_headers)
+                      void *cls)
 {
   struct MHD_Daemon *daemon = connection->daemon;
   char *equals;
   char *amper;
 
-  *num_headers = 0;
   while ( (NULL != args) &&
           ('\0' != args[0]) )
   {
@@ -219,14 +193,13 @@ MHD_parse_arguments_ (struct MHD_Connection *connection,
         key_len = daemon->unescape_callback (daemon->unescape_callback_cls,
                                              connection,
                                              args);
-        if (MHD_NO == cb (connection,
+        if (MHD_NO == cb (cls,
                           args,
                           key_len,
                           NULL,
                           0,
                           kind))
           return MHD_NO;
-        (*num_headers)++;
         break;
       }
       /* got 'foo=bar' */
@@ -240,14 +213,13 @@ MHD_parse_arguments_ (struct MHD_Connection *connection,
       value_len = daemon->unescape_callback (daemon->unescape_callback_cls,
                                              connection,
                                              equals);
-      if (MHD_NO == cb (connection,
+      if (MHD_NO == cb (cls,
                         args,
                         key_len,
                         equals,
                         value_len,
                         kind))
         return MHD_NO;
-      (*num_headers)++;
       break;
     }
     /* amper is non-NULL here */
@@ -261,7 +233,7 @@ MHD_parse_arguments_ (struct MHD_Connection *connection,
       key_len = daemon->unescape_callback (daemon->unescape_callback_cls,
                                            connection,
                                            args);
-      if (MHD_NO == cb (connection,
+      if (MHD_NO == cb (cls,
                         args,
                         key_len,
                         NULL,
@@ -269,7 +241,6 @@ MHD_parse_arguments_ (struct MHD_Connection *connection,
                         kind))
         return MHD_NO;
       /* continue with 'bar' */
-      (*num_headers)++;
       args = amper;
       continue;
     }
@@ -285,14 +256,13 @@ MHD_parse_arguments_ (struct MHD_Connection *connection,
     value_len = daemon->unescape_callback (daemon->unescape_callback_cls,
                                            connection,
                                            equals);
-    if (MHD_NO == cb (connection,
+    if (MHD_NO == cb (cls,
                       args,
                       key_len,
                       equals,
                       value_len,
                       kind))
       return MHD_NO;
-    (*num_headers)++;
     args = amper;
   }
   return MHD_YES;
