@@ -1,6 +1,6 @@
 /*
   This file is part of libmicrohttpd
-  Copyright (C) 2019 Karlson2k (Evgeny Grin)
+  Copyright (C) 2019-2023 Evgeny Grin (Karlson2k)
 
   This test tool is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
@@ -24,11 +24,16 @@
  */
 
 #include "mhd_options.h"
-#include "sha256.h"
+#include "mhd_sha256_wrap.h"
 #include "test_helpers.h"
 #include <stdio.h>
 #include <stdlib.h>
 
+#if defined(MHD_SHA256_TLSLIB) && defined(MHD_HTTPS_REQUIRE_GCRYPT)
+#define NEED_GCRYP_INIT 1
+#include <gcrypt.h>
+#endif /* MHD_SHA256_TLSLIB && MHD_HTTPS_REQUIRE_GCRYPT */
+
 static int verbose = 0; /* verbose level (0-1)*/
 
 
@@ -82,12 +87,22 @@ static const struct data_unit1 data_units1[] = {
     0x73, 0x82, 0xac, 0x92,
     0xca, 0x37, 0xfd, 0x72, 0x8b, 0x0c, 0xd1, 0x6c, 0x55, 0xd5, 0x88, 0x98,
     0x24, 0xfa, 0x16, 0xf2}},
-  {D_STR_W_LEN ("abcdefghijklmnopqrstuvwxyzzyxwvutsrqponMLKJIHGFEDCBA"
+  {D_STR_W_LEN ("abcdefghijklmnopqrstuvwxyzzyxwvutsrqponMLKJIHGFEDCBA" \
                 "abcdefghijklmnopqrstuvwxyzzyxwvutsrqponMLKJIHGFEDCBA"),
    {0x27, 0xd1, 0xe8, 0xbc, 0x6a, 0x79, 0x16, 0x83, 0x61, 0x73, 0xa9, 0xa8,
     0x9b, 0xaf, 0xaf, 0xcf,
     0x47, 0x4d, 0x09, 0xef, 0x6d, 0x50, 0x35, 0x12, 0x25, 0x72, 0xd8, 0x68,
     0xdc, 0x1f, 0xd2, 0xf4}},
+  {D_STR_W_LEN ("/long/long/long/long/long/long/long/long/long/long/long" \
+                "/long/long/long/long/long/long/long/long/long/long/long" \
+                "/long/long/long/long/long/long/long/long/long/long/long" \
+                "/long/long/long/long/long/long/long/long/long/long/long" \
+                "/long/long/long/long/long/long/long/long/long/long/long" \
+                "/long/long/long/long/long/long/long/long/long/long/long" \
+                "/long/long/long/long/path?with%20some=parameters"),
+   {0x73, 0x85, 0xc5, 0xb9, 0x8f, 0xaf, 0x7d, 0x5e, 0xad, 0xd8, 0x0b, 0x8e,
+    0x12, 0xdb, 0x28, 0x60, 0xc7, 0xc7, 0x55, 0x05, 0x2f, 0x7c, 0x6f, 0xfa,
+    0xd1, 0xe3, 0xe1, 0x7b, 0x04, 0xd4, 0xb0, 0x21}}
 };
 
 static const size_t units1_num = sizeof(data_units1) / sizeof(data_units1[0]);
@@ -310,9 +325,9 @@ check_result (const char *test_name,
   {
     char calc_str[SHA256_DIGEST_STRING_SIZE];
     bin2hex (calculated, SHA256_DIGEST_SIZE, calc_str);
-    printf (
-      "PASSED: %s check %u: calculated digest %s matches expected digest.\n",
-      test_name, check_num, calc_str);
+    printf ("PASSED: %s check %u: calculated digest %s "
+            "matches expected digest.\n",
+            test_name, check_num, calc_str);
     fflush (stdout);
   }
   return failed ? 1 : 0;
@@ -329,19 +344,27 @@ test1_str (void)
 {
   int num_failed = 0;
   unsigned int i;
+  struct Sha256CtxWr ctx;
 
+  MHD_SHA256_init_one_time (&ctx);
   for (i = 0; i < units1_num; i++)
   {
-    struct sha256_ctx ctx;
     uint8_t digest[SHA256_DIGEST_SIZE];
 
-    MHD_SHA256_init (&ctx);
     MHD_SHA256_update (&ctx, (const uint8_t *) data_units1[i].str_l.str,
                        data_units1[i].str_l.len);
-    MHD_SHA256_finish (&ctx, digest);
+    MHD_SHA256_finish_reset (&ctx, digest);
+#ifdef MHD_SHA256_HAS_EXT_ERROR
+    if (0 != ctx.ext_error)
+    {
+      fprintf (stderr, "External hashing error: %d.\n", ctx.ext_error);
+      exit (99);
+    }
+#endif /* MHD_SHA256_HAS_EXT_ERROR */
     num_failed += check_result (MHD_FUNC_, i, digest,
                                 data_units1[i].digest);
   }
+  MHD_SHA256_deinit (&ctx);
   return num_failed;
 }
 
@@ -351,19 +374,27 @@ test1_bin (void)
 {
   int num_failed = 0;
   unsigned int i;
+  struct Sha256CtxWr ctx;
 
+  MHD_SHA256_init_one_time (&ctx);
   for (i = 0; i < units2_num; i++)
   {
-    struct sha256_ctx ctx;
     uint8_t digest[SHA256_DIGEST_SIZE];
 
-    MHD_SHA256_init (&ctx);
     MHD_SHA256_update (&ctx, data_units2[i].bin_l.bin,
                        data_units2[i].bin_l.len);
-    MHD_SHA256_finish (&ctx, digest);
+    MHD_SHA256_finish_reset (&ctx, digest);
+#ifdef MHD_SHA256_HAS_EXT_ERROR
+    if (0 != ctx.ext_error)
+    {
+      fprintf (stderr, "External hashing error: %d.\n", ctx.ext_error);
+      exit (99);
+    }
+#endif /* MHD_SHA256_HAS_EXT_ERROR */
     num_failed += check_result (MHD_FUNC_, i, digest,
                                 data_units2[i].digest);
   }
+  MHD_SHA256_deinit (&ctx);
   return num_failed;
 }
 
@@ -374,23 +405,34 @@ test2_str (void)
 {
   int num_failed = 0;
   unsigned int i;
+  struct Sha256CtxWr ctx;
 
+  MHD_SHA256_init_one_time (&ctx);
   for (i = 0; i < units1_num; i++)
   {
-    struct sha256_ctx ctx;
     uint8_t digest[SHA256_DIGEST_SIZE];
     size_t part_s = data_units1[i].str_l.len / 4;
 
-    MHD_SHA256_init (&ctx);
+    MHD_SHA256_update (&ctx, (const uint8_t *) "", 0);
     MHD_SHA256_update (&ctx, (const uint8_t *) data_units1[i].str_l.str,
                        part_s);
+    MHD_SHA256_update (&ctx, (const uint8_t *) "", 0);
     MHD_SHA256_update (&ctx, (const uint8_t *) data_units1[i].str_l.str
                        + part_s,
                        data_units1[i].str_l.len - part_s);
-    MHD_SHA256_finish (&ctx, digest);
+    MHD_SHA256_update (&ctx, (const uint8_t *) "", 0);
+    MHD_SHA256_finish_reset (&ctx, digest);
+#ifdef MHD_SHA256_HAS_EXT_ERROR
+    if (0 != ctx.ext_error)
+    {
+      fprintf (stderr, "External hashing error: %d.\n", ctx.ext_error);
+      exit (99);
+    }
+#endif /* MHD_SHA256_HAS_EXT_ERROR */
     num_failed += check_result (MHD_FUNC_, i, digest,
                                 data_units1[i].digest);
   }
+  MHD_SHA256_deinit (&ctx);
   return num_failed;
 }
 
@@ -400,21 +442,30 @@ test2_bin (void)
 {
   int num_failed = 0;
   unsigned int i;
+  struct Sha256CtxWr ctx;
 
+  MHD_SHA256_init_one_time (&ctx);
   for (i = 0; i < units2_num; i++)
   {
-    struct sha256_ctx ctx;
     uint8_t digest[SHA256_DIGEST_SIZE];
     size_t part_s = data_units2[i].bin_l.len * 2 / 3;
 
-    MHD_SHA256_init (&ctx);
     MHD_SHA256_update (&ctx, data_units2[i].bin_l.bin, part_s);
+    MHD_SHA256_update (&ctx, (const uint8_t *) "", 0);
     MHD_SHA256_update (&ctx, data_units2[i].bin_l.bin + part_s,
                        data_units2[i].bin_l.len - part_s);
-    MHD_SHA256_finish (&ctx, digest);
+    MHD_SHA256_finish_reset (&ctx, digest);
+#ifdef MHD_SHA256_HAS_EXT_ERROR
+    if (0 != ctx.ext_error)
+    {
+      fprintf (stderr, "External hashing error: %d.\n", ctx.ext_error);
+      exit (99);
+    }
+#endif /* MHD_SHA256_HAS_EXT_ERROR */
     num_failed += check_result (MHD_FUNC_, i, digest,
                                 data_units2[i].digest);
   }
+  MHD_SHA256_deinit (&ctx);
   return num_failed;
 }
 
@@ -430,9 +481,11 @@ test_unaligned (void)
   unsigned int offset;
   uint8_t *buf;
   uint8_t *digest_buf;
+  struct Sha256CtxWr ctx;
 
   const struct data_unit2 *const tdata = data_units2 + DATA_POS;
 
+  MHD_SHA256_init_one_time (&ctx);
   buf = malloc (tdata->bin_l.len + MAX_OFFSET);
   digest_buf = malloc (SHA256_DIGEST_SIZE + MAX_OFFSET);
   if ((NULL == buf) || (NULL == digest_buf))
@@ -440,7 +493,6 @@ test_unaligned (void)
 
   for (offset = MAX_OFFSET; offset >= 1; --offset)
   {
-    struct sha256_ctx ctx;
     uint8_t *unaligned_digest;
     uint8_t *unaligned_buf;
 
@@ -449,14 +501,21 @@ test_unaligned (void)
     unaligned_digest = digest_buf + MAX_OFFSET - offset;
     memset (unaligned_digest, 0, SHA256_DIGEST_SIZE);
 
-    MHD_SHA256_init (&ctx);
     MHD_SHA256_update (&ctx, unaligned_buf, tdata->bin_l.len);
-    MHD_SHA256_finish (&ctx, unaligned_digest);
+    MHD_SHA256_finish_reset (&ctx, unaligned_digest);
+#ifdef MHD_SHA256_HAS_EXT_ERROR
+    if (0 != ctx.ext_error)
+    {
+      fprintf (stderr, "External hashing error: %d.\n", ctx.ext_error);
+      exit (99);
+    }
+#endif /* MHD_SHA256_HAS_EXT_ERROR */
     num_failed += check_result (MHD_FUNC_, MAX_OFFSET - offset,
                                 unaligned_digest, tdata->digest);
   }
   free (digest_buf);
   free (buf);
+  MHD_SHA256_deinit (&ctx);
   return num_failed;
 }
 
@@ -469,6 +528,13 @@ main (int argc, char *argv[])
   if (has_param (argc, argv, "-v") || has_param (argc, argv, "--verbose"))
     verbose = 1;
 
+#ifdef NEED_GCRYP_INIT
+  gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
+#ifdef GCRYCTL_INITIALIZATION_FINISHED
+  gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
+#endif /* GCRYCTL_INITIALIZATION_FINISHED */
+#endif /* NEED_GCRYP_INIT */
+
   num_failed += test1_str ();
   num_failed += test1_bin ();
 
